love.filesystem.setIdentity("roppongiplus")

SEGMENTS = 50

startX = {}
startX[1] = 304
startX[2] = 499
startX[3] = 272
startX[4] = 454
startX[5] = 554
startX[6] = 502
startX[7] = 243
startX[8] = 454
startX[9] = 384
startX[10] = 758

barrierlimit = {}
barrierlimit[1] = {0, 0}
barrierlimit[2] = {466, 768}
barrierlimit[3] = {664, 768}
barrierlimit[4] = {768, 768}
barrierlimit[5] = {0, 450}
barrierlimit[6] = {550, 540}
barrierlimit[7] = {0, 275}
barrierlimit[8] = {592, 330}
barrierlimit[9] = {768, 768}
barrierlimit[10] = {0, 0}

function love.load()
  level = 1
  length = 128
  g = 600
  loadLevel(level)
  love.graphics.setLineWidth(4)
end

function love.update(dt)
  if running then
    if love.mouse.isDown("l") and love.mouse.isDown("r") then
      vx = 0
      vy = 0
      vr = 0
    elseif love.mouse.isDown("l") then
      vr = vr + dt * g * math.cos(r) / length
      r = r + vr * dt
      x2 = x1 + math.cos(r) * length
      y2 = y1 + math.sin(r) * length
      x = (x1 + x2) / 2
      y = (y1 + y2) / 2
      vx = math.cos(r + math.pi/2) * vr * length / 2
      vy = math.sin(r + math.pi/2) * vr * length / 2
    elseif love.mouse.isDown("r") then
      vr = vr - dt * g * math.cos(r) / length
      r = r + vr * dt
      x1 = x2 - math.cos(r) * length
      y1 = y2 - math.sin(r) * length
      x = (x1 + x2) / 2
      y = (y1 + y2) / 2
      vx = -math.cos(r + math.pi/2) * vr * length / 2
      vy = -math.sin(r + math.pi/2) * vr * length / 2
    else
      vy = vy + g * dt
      x = x + vx * dt
      y = y + vy * dt
      r = r + vr * dt
      x1 = x - math.cos(r) * length/2
      y1 = y - math.sin(r) * length/2
      x2 = x + math.cos(r) * length/2
      y2 = y + math.sin(r) * length/2
    end
    
    if not isBGColor(imagePixel(x1, y1)) or not isBGColor(imagePixel(x2, y2)) or outOfBounds(x1, y1) or outOfBounds(x2, y2) then
      loadLevel(level)
    elseif y1 > love.graphics.getHeight() or y2 > love.graphics.getHeight() then
      if level < 10 then level = level + 1
      else level = 1
      end
      loadLevel(level)
    end
  end
end

function love.draw()
  love.graphics.setColor(255, 255, 255, 255)
  love.graphics.draw(levelimage, 0, 0)
  dx = x2 - x1
  dy = y2 - y1
  for i = 1, SEGMENTS do
    love.graphics.setColor(255, (i-1)/(SEGMENTS-1) * 255, 0, 255)
    love.graphics.line(x1 + 0.99 * (i-1)/SEGMENTS * dx, y1 + 0.99 * (i-1)/SEGMENTS * dy, x1 + 1.01 * i/SEGMENTS * dx, y1 + 1.01 * i/SEGMENTS * dy)
  end
end

function love.mousepressed(x, y, button)
  running = true
  if button == "l" then
    if not love.mouse.isDown("r") then
      vx2 = vx - math.cos(r - math.pi/2) * vr * length / 2
      vy2 = vy - math.sin(r - math.pi/2) * vr * length / 2
      speed = math.sqrt(vx2^2 + vy2^2)
      angle = math.atan2(vy2, vx2)
      vr = math.sin(angle - r) * speed / length
    end
  elseif button == "r" then
    if not love.mouse.isDown("l") then
      vx1 = vx - math.cos(r + math.pi/2) * vr * length / 2
      vy1 = vy - math.sin(r + math.pi/2) * vr * length / 2
      speed = math.sqrt(vx1^2 + vy1^2)
      angle = math.atan2(vy1, vx1)
      vr = - math.sin(angle - r) * speed / length
    end
  end
end

function love.keypressed(k, u)
  if k == "escape" then
    love.event.push("q")
  end
end

function love.keyreleased(k)
end

function loadLevel(level)
  levelimage = love.graphics.newImage("level" .. level .. ".png")
  imagedata = love.image.newImageData("level" .. level .. ".png")
  bgcolor = {imagedata:getPixel(0, 0)}
  x1 = startX[level]
  y1 = 16
  x2 = x1 + length
  y2 = 16
  x  = x1 + length/2
  y  = 16
  r = 0
  vx = 0
  vy = 0
  vr = 0
  running = false
end

function imagePixel(x, y)
  if x >= 0 and x < love.graphics.getWidth() - 1 and y >= 0 and y < love.graphics.getHeight() - 1 then
    return imagedata:getPixel(x, y)
  else
    return bgcolor[1], bgcolor[2], bgcolor[3], bgcolor[4]
  end
end

function isBGColor(r, g, b, a)
  return r == bgcolor[1] and g == bgcolor[2] and b == bgcolor[3]
end

function outOfBounds(x, y)
  return x < 0 and y < barrierlimit[level][1] or x > love.graphics.getWidth() and y < barrierlimit[level][2]
end
